// -*-C++-*-
//
// File:      DopeVector.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Aug  9 16:47:44 MEST 2002

#ifndef DopeVector_h
#define DopeVector_h
#include <f90cxx.h>

/************************************************************
*  Templated interface to F90 vectors of a particular type.
*
*************************************************************/


template<typename T, int N = 1>
struct DopeVector
{
public:


    DopeVector() : addr(0) { }

    DopeVector(const T* vec, const unsigned long* ext, const unsigned long* low=0) 
    {
        addr = const_cast<T*>(vec);		// address to first element
        offset = 0;
      

        // Swap C++ and Fortran indices
        for (int i = N-1, j = 0; i >= 0; --i, ++j) 
        {
            dim[j].extent	= ext[i];
            dim[j].lower	= (low ? low[i] : 0);

            if (j == 0)
                dim[j].mult = sizeof(T);
            else
                dim[j].mult = dim[j-1].mult * dim[j-1].extent;

            offset -= dim[j].mult * dim[j].lower;
        }
       

    }    
  
    DopeVector(const T* vec, const std::vector<unsigned long>& ext,
	    const std::vector<unsigned long>& low=std::vector<unsigned long>( 0) ) 
    {
      

        assert(ext.size() == N);
        assert(low.size() == 0 || low.size() == N);

        addr = const_cast<T*>(vec);		// address to first element
        offset = 0;

        // Swap C++ and Fortran indices
        for (int i = N-1, j = 0; i >= 0; --i, ++j) 
        {
            dim[j].extent	= ext[i];
            dim[j].lower	= (low.size() > 0 ? low[i] : 0);

            if (j == 0)
                dim[j].mult = sizeof(T);
            else
                dim[j].mult = dim[j-1].mult * dim[j-1].extent;

            offset -= dim[j].mult * dim[j].lower;
        }

    

    }

    T* addr;
  

    Integer3 offset;
    Triplet dim[1];

    int length() const
    {
     

        return dim[0].extent;
    }

    int lbound() const
    {
    


        return dim[0].lower;
    }

    int ubound() const
    {


   

        return dim[0].lower+dim[0].extent-1;
    }

    T& operator[]( unsigned long idx ) const
    {
 

      

        return *(T*)((int)addr + (dim[0].mult*idx + offset));
    }

    int sizeOfType() const {
   


        return sizeof(T);
    }

private:


};

#endif
