// -*-C++-*-
//
// File:      OmSourceLog.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Wed Oct  2 15:31:02 CEST 2002
// Usage:     -
// Info:      -

#include "OmSourceLog.h"
#include "OmSourceParameters.h"
#include <qstring.h>
#include <qtooltip.h>
#include<qfontmetrics.h>
#include <qheader.h>

/*****************************************************
*
*****************************************************/
OmSourceLog::OmSourceLog(QWidget* parent, const char* name) : QTable(parent, name)
{
    int noColumns = 8;

    setReadOnly(true);   // Can only read the table
    setSelectionMode(QTable::SingleRow);
    setNumCols( noColumns);
    setNumRows(1);
    
    QHeader *th = this->horizontalHeader();
 
    th->setLabel(0, "Aperture");
 
    th->setLabel(1,"Inner Bgr");
 
    th->setLabel(2,"Outer Bgr");

    th->setLabel(3,"Axial ratio");
 
    th->setLabel(4,"Pos angle");
 

    th->setLabel(5,"Magnitude");
 

    th->setLabel(6,"  +/-");

    th->setLabel(7,"Background subtraction method                 ");


 

    setText(0, 0,"Aperture");

    setText(0, 1,"Inner Bgr");
 
    setText(0, 2,"Outer Bgr");
  
    setText(0, 3,"Axial ratio");
 

    setText(0, 4,"Pos angle");
 

    setText(0, 5,"Magnitude");
 

    setText(0, 6," +/- ");
 
    setText(0, 7,"                                                          Background");
 




      
    QFontMetrics fm = fontMetrics();

 


    int maxWidth = 0;
    /********************************************
    * Adjust the size of the cells
    *******************************************/
    for (int i = 0; i < noColumns; i++)
    {   
        int width =   fm.width( (const char*) text(0, i)) + 7;
        setColumnWidth(i,  width );
        maxWidth += width;

    }
    resize( maxWidth, height() );
    setNumRows(0);
    
}
/*****************************************************
*
*****************************************************/
OmSourceLog::~OmSourceLog()
{
}
/*****************************************************
*
*****************************************************/
void OmSourceLog::add(const OmSourceParameter& s)
{
    int rowNo = numRows();

    setNumRows(rowNo + 1);
    int colNo = 0;

    QString text;
   
					

    if (s.photometryradius == 0.0)
    {

        setText(rowNo, colNo++, "OMDETECT");

              
        setText(rowNo, colNo++, "---");
        setText(rowNo, colNo++, "---");
        setText(rowNo, colNo++, "---");
        setText(rowNo, colNo++, "---");


    }
    else
    {

        setText(rowNo, colNo++, text.sprintf("%2d", int(s.photometryradius)));   
        setText(rowNo, colNo++, text.sprintf("%2d", int(s.innerbackgroundradius)));
        setText(rowNo, colNo++, text.sprintf("%2d", int(s.outerbackgroundradius)));
        setText(rowNo, colNo++, text.sprintf("%4.2f", s.axialRatio));
        setText(rowNo, colNo++, text.sprintf("%3d", int(s.alpha)));
    }



    setText(rowNo, colNo++,  text.sprintf("%5.2f", s.magnitude));
    setText(rowNo, colNo++,  text.sprintf("%5.2f", s.magnitudeerror));



    //   cout << "OmsourceLog.cc line 145 s.iMin=" << s.iMin << endl;

    switch( s.iMin )
    {
        case( 0):
	  
            setText(rowNo, colNo++,  "Subtracted using omdetect background map");
	    break;

        case( -2 ):
            setText(rowNo, colNo++,  "Interpolation using background annulus");
	    break;
        case( -1 ):
            setText(rowNo, colNo++,  "Mean background computed from annulus pixels");
	    break;
        case( -3 ):
	    
            text.sprintf("Used fixed background of %5.1f counts/per pixel", s.rawbackgroundcountrateerror);
            setText(rowNo, colNo++,  text);
	    break;
        case( -4 ):
            setText(rowNo, colNo++,  "Used background computed from PSF fit");
	    break;


        
    }



    QToolTip::add( this , QString("This table stores the photometry parameters used to compute the instrumental magnitude for the selected source\n")+
		          QString("Column 1 = For point-sources the aperture radius used to compute the source counts\n") +
		          QString("           For extended-sources the semi-minor axis of the elliptical annulus used\n") +
		          QString(   "           to compute the source counts\n") +    
		          QString(   "           OMDETECT means that the pixels assigned by omdetect to the source were used\n") +
		          QString(   "           to compute the magnitude (The background was subtracted using the background\n") +
		          QString(   "           map computed by omdetect)\n") +
		          QString(   "Column 2 = For point-sources the inner background annulus radius used for computing\n") +
		          QString(   "           the mean background\n") + 
		          QString(   "           For extended-sources the semi-minor axis of the inner background annulus\n") +
		          QString(   "           used to compute the background\n") +
		          QString(   "Column 3 = For point-sources the outer background annulus radius used for computing\n") +
		          QString(   "           the mean background\n") + 
		          QString(   "           For extended-sources the semi-minor axis of the outer background annulus\n") +
			  QString(   "           used to compute the background\n") +                       
			  QString(   "Column 4 = For extended-sources only, the axial ratio of the annulii ellipses\n") +
			  QString(   "           (semi-major axis = semi-minor axis x axial ratio)\n") +
			  QString(   "Column 5 = For extended-sources only, the position angle of ellipse\n") +
			  QString(   "           (measured anti-clockwise from x-origin)\n") +
			  QString(   "Column 6 = instrumental magnitude\n") +
			  QString(   "Column 7 = instrumental magnitude error\n") +
			  QString(   "Column 8 = For extended-sources only- the method used to compute the background\n") +
			  QString(   "           0=using the background map computed by omdetect\n") +
			  QString(   "           1=using the background annulus to compute the mean background\n") +
			  QString(   "           2=using the pixels within the background annulus to interpolate\n") +
                          QString(   "           the background to the source region")); 
                         


}
/*****************************************************
*
*****************************************************/
QString OmSourceLog::getLine(const int index)
{
    QString line;
    for (int i = 0; i < numCols(); i++)
        line += text(index, i) + " ";

    return line;
}
/*****************************************************
*
*****************************************************/
void OmSourceLog::restart()
{
    setNumRows(0);
}
















