#include "OmSourceTableDlg.h"
#include <qapplication.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include "OmSourceVersionNo.h"
#include "OmSourceLog.h"
#include "OmSourceGroupBox.h"
#include "OmSourceParameters.h"
#include "OmSourceButton.h"
#include "AngleFormatter.h"
#include <qtable.h>
#include <qstring.h>
#include <qtooltip.h>
#include <qdesktopwidget.h>
#include <qlayout.h>
#include <qprinter.h> 
#include <qpainter.h>
#include <qfont.h> 
#include <qpaintdevicemetrics.h> 
#include <iostream>
#include<cmath>

/********************************************************************************
 * Dialog for running omdetect- allows you to change the parameters nsigma
 * and minsignificance.          
 *
 *******************************************************************************/
F90MODPROCPROTO(void,omsource_glue, deletelist)();
extern F90MODVARDEF(OmSourceParameters, omsource_glue, mysources);
TableDlg::TableDlg(QWidget *parent) :
  QDialog(parent, "", true)
{

   
  // The table that displays the sources data    
    _table = new QTable(10, 3, this, 0);
   


    QDesktopWidget screen;
    QRect rect = screen.screenGeometry();

   _table->setGeometry(0, 0, rect.right(), (rect.bottom() - rect.top()) / 2);

    QToolTip::add(_table, "This table displays various parameters\n"
                          "for each source in the current list");
   
    _table->setCaption("SOURCE-LIST TABLE");
 

    connect( _table, SIGNAL( clicked(int,int,int,const QPoint&) ),
                   this, SLOT( selectionChanged(int, int) ) );
    connect( _table, SIGNAL( pressed(int,int,int,const QPoint&) ),
                   this, SLOT( selectionChanged(int, int) ) );

     connect( _table, SIGNAL( currentChanged(int,int) ),
                   this, SLOT( selectionChanged(int, int) ) );

   




    int noColumns = 8;  // No of columns in table

    _table->setReadOnly(true);   // Can only read the table
    _table->setNumCols( noColumns);
    OmSourceParameters& sources = F90MODVAR(omsource_glue, mysources); 
  
    _table->setNumRows(sources.ubound());  

    QHeader *th = _table->horizontalHeader();
  
     
    th->setLabel(0, "RIGHT ASCENSION");
    _table->setText(0, 0, "RIGHT ASCENSION");
 

    th->setLabel(1, "DECLINATION");
    _table->setText(0, 1, "DECLINATION");
 

    th->setLabel(2,"PHOTOMETRY\nRADIUS\npixels");

    _table->setText(0, 2, "PHOTOMETRY");
 



    th->setLabel(3,"CORRECTED\nSOURCE\nCOUNT RATE\ncounts/sec");
    _table->setText(0, 3, "CORRECTED");


    th->setLabel(4,"+/-");
    _table->setText(0, 4, "counts");
  


    th->setLabel(5,"CORRECTED\nBACKGROUND\nCOUNT RATE\ncounts/sec");
    _table->setText(0, 5, "CORRECTED");


    //    th->setLabel(6,"+/-");
    //    _table->setText(0, 6, "counts");
  
    th->setLabel(6,"INSTRUMENTAL\nMAGNITUDE");
    _table->setText(0, 6, "INSTRUMENTAL");
  
    th->setLabel(7,"+/-");
    _table->setText(0, 7, "+/-");
  

    /********************************************
    * Adjust the size of the cells
    *******************************************/

    QFont f = font();
      
    QFontMetrics fm( f );
 
 
    int width = 0;
    for (int i = 0; i < noColumns; i++)
    {   
      //        int columnWidth =  _table->text(0, i).length() * 10;
        int columnWidth =  fm.width(_table->text(0, i)) + 12;

        _table->setColumnWidth(i, columnWidth);
        width += columnWidth;  
  
        _table->setText(0, i, " ");


   }
    // Adjust table width
    width += 50;

    rect = _table->rect();
 
   _table->setGeometry(0, 0, width, (rect.bottom() - rect.top()) / 2);

    rect = _table->rect();
 

 
    OmSourceGroupBox *group = new OmSourceGroupBox( this, 
          "Source list operations", (rect.right() - rect.left()) / 2 - 150, 
                  rect.bottom() + 50, 200, 80, 
                 "An operation that is performed on the source-list" );   
						  

    QHBoxLayout* layout = new QHBoxLayout(group);
    
    layout->setMargin(10);
    layout->setSpacing(10);  
   


    _printSourceListBtn = new OmSourceButton(group, "Print",
                "Click here to print the contents of the source list" );
  


    layout->addWidget( _printSourceListBtn);
    connect(_printSourceListBtn, SIGNAL(clicked()), this, SLOT (printTable()) );
   
 
          
    _deleteListBtn = new OmSourceButton(group, "Delete",
               "Click here to delete the source list");
  
    layout->addWidget( _deleteListBtn);
  
    connect(_deleteListBtn, SIGNAL(clicked()), this, SLOT (deleteList()) );


    // LOad the table  
    load(); 


    QPushButton* okButton = new QPushButton("OK", this);
    okButton->setGeometry((rect.right() - rect.left()) / 2 - 50, 
                  rect.bottom() + 180, 100, 30);


    connect(okButton, SIGNAL( clicked() ), this, SLOT( accept() ));



    //    connect(okButton, SIGNAL( clicked() ), this, SLOT( end() ));
 
  

    
};
/*********************************************************
*
*********************************************************/
void TableDlg::load()
{
 

  
  
    // Load the sources
     
    AngleFormatter ra;
    ra.setMode(AngleFormatter::HMS);
    AngleFormatter dec;
    dec.setMode(AngleFormatter::DMS);
    OmSourceParameters& sources = F90MODVAR(omsource_glue, mysources); 
   
 
    for (int i = sources.lbound(); i <= sources.ubound(); i++) 
    {

             
          QString text;
          
          int k = i - 1;
          _table->setText(k, 0, ra.asString(sources[i].rightascension).c_str());
                
          _table->setText(k, 1, dec.asString(sources[i].declination).c_str());
       

	  text.sprintf("%2d",  (int)sources[i].photometryradius);
	 _table->setText(k, 2, text);


	
	  text.sprintf("%8.2f", sources[i].corrcountrate);  
	  _table->setText(k, 3, text);
	
          text.sprintf("%8.2f", sqrt(sources[i].corrcountrateerror));  
	  _table->setText(k, 4, text);
	        

          text.sprintf("%8.2f", sources[i].corrbackgroundcountrate);  
	  _table->setText(k, 5, text);


	  //	  text.sprintf("%8.2f", sqrt(sources[i].corrbackgroundcountrateerror));  
	  // _table->setText(k, 6, text);
	  
	       	 
          text.sprintf("%8.2f", sources[i].magnitude);  
	  _table->setText(k, 6, text);
	
            	 
          text.sprintf("%8.2f", sources[i].magnitudeerror);  
	  _table->setText(k, 7, text);
	


    }    

     
}
/**************************************************
*
*************************************************/
void TableDlg::selectionChanged(int row, int)
{
    // Tell parent that the sourceselection has changed 
    emit selectionChanged(row);
} 
 
/*************************************************************
 * deleteList
 * Purpose - deletes all the sources from the list
 ************************************************************/
void TableDlg::deleteList()
{
  

    F90MODPROC(omsource_glue, deletelist)();
    
    _table->setNumRows(0);   
     emit deleteSourceList();   
      
  
   
}
/****************************************************
*
****************************************************/
void TableDlg::printTable()
{
    print(this);

}
/*********************************************************
* printSourceList
* Prints the contents of the combo box that stores
* the average magnitudes
*********************************************************/
void TableDlg::Print(QPrinter* printer, QPainter* p,  
               QFontMetrics* fm,  QPaintDeviceMetrics* metrics)
{
    const int MARGIN = metrics->logicalDpiX() / 2; // half-inch margin
    int yPos        = MARGIN;               // y position for each line


    //    printHeading(p,  fm, metrics, MARGIN, yPos);







    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, "                 Source list");
    yPos += 2 * fm->lineSpacing();


    QHeader *th = _table->horizontalHeader();
  
    int position = MARGIN;
    for (int j = 0; j < _table->numCols(); j++)
    {
        p->drawText(position, yPos, metrics->width() - 2 * MARGIN,
	                     fm->lineSpacing(), ExpandTabs, th->label(j));
        position += _table->columnWidth(j);
    }
     
    yPos += 2 * fm->lineSpacing();
  
 
    
    for (int i = 0; i < _table->numRows(); i++)
    {   
      
      //        _table->setColumnWidth(i, _table->text(0, i).length() * 10);
          
 
      if ( printer->aborted() )
                 break;
        if ( yPos + fm->lineSpacing() > metrics->height() - MARGIN ) 
        {
                // no more room on this page
	        if ( !printer->newPage() )          // start new page
		    break;                          // some error
                yPos = MARGIN;                      // back to top of page
        }
        position = MARGIN;
        for (int j = 0; j < _table->numCols(); j++)
	{
	    p->drawText(position, yPos, metrics->width() - 2 * MARGIN,
	                     fm->lineSpacing(), ExpandTabs, _table->text(i, j));
            position += _table->columnWidth(j);
        }
        yPos += fm->lineSpacing();
    }

}







